function [D, ftsout, ang, ind] = computeDescriptors(Im,fts,DESCRIPTOR_BIN)
% function [D,interestPoints,angles,ind] = compute_descr(img,interestPoints,DESCRIPTOR_BIN)
%
% Compute SIFT descriptors at interest points using K. Mikolajczyk's
% binary.
%
% Inputs:
% img - Input image.
% interestPoints - (5 x n), where each column is [x y a b c]:
%       x,y is position of the feature.
%       a,b,c are parameters of 2x2 symmetric matrix A=[a b; b c] 
%       representing local affine transformation (no rotation) taking
%       elliptical neighbourhood of x,y onto unit circle. abc must be
%       normalized so that det(A)=1., see the note 4).
% DESCRIPTOR_BIN - Path to descriptor binary
%
% Outputs:
% D - n x d matrix where each row contains d-dimensional SIFT descriptor
%   valued 0..255
% interestPoints - [x y a b c]  region parameters (one region per column)
%   for each descriptor in D
% angles ... detected dominant gradient orientation for each descriptor in D
% ind - index array, one element for each descriptor in D, pointing to
%   corresponding feature in input innterestPoints (see the notes below)
%
% Notes:
%              
%           1) In km's binary, each elliptical feature is 5 x enlarged
%              before computing the descriptor.
%              To keep the original scale (specified by 'scale'
%              parameter), we divide 'scale' parameter by 5 before we
%              input the features into the binary.
%           
%           2) If you display ellipses overlaid over the input image using
%           '-DR' option for the KM's binary, the ellipses are scaled only 3x. 
%              (Not 5x as  used for descriptor computation);
%
%           3) Transformation matrix A=[a b;b c] (input to the binary) 
%              must be normalized so that its determinant is 1.
%              The scale factor input into km binary should then be
%              "scale = s_rel*sqrt(det(A))",
%              where s_rel is an additional (and optional) scale multipler 
%              specified in "P.params.words.descr_scale"
%
% Note:   The output descriptor array D may be either longer or shorter
% then the input interestPoints vector:
%            1) One feature can have multiple descriptors corresponding to 
%               multiple detected dominant gradient orientations due to
%               e.g. bimodal gradient orientation histogram.
%            2) Some features can have no descriptor at all, e.g. feature
%            ellipses reaching out of image
%                         
% Josef Sivic <josef@robots.ox.ac.uk>
% 13/12/2002
%
% Modified by Bryan Russell <brussell@csail.mit.edu>

  descr_type = 'sift';
  scale      = 1;
  verbose    = 0;

  % initialize input
  D          = [];
  ftsout     = [];
  ang        = [];
  ind        = [];
  
  % if no input return
  if isempty(fts)
    return;
  end;   

  tname = tempname;
  image_file = [tname '.pgm'];
  corner_file = [tname '.in'];
  out_file = [tname '.desc'];
  
  % save image as pgm
  if max(Im(:))<2
    vgg_save_image_pgm(image_file,Im*255);
  else
    vgg_save_image_pgm(image_file,Im);
  end

  % create input feature point file 

  % values we do not use, but that are set as suggested by KM
  cornerness = 1000 * ones(size(fts,2),1);    % ignore
  angle      = 0 * ones(size(fts,2),1);       % Make SIFT rotation !variant! Angle must be set to zero 
  type       = 0  * ones(size(fts,2),1);      % ignore
  lap        = 10 * ones(size(fts,2),1);      % ignore
  extr       = 0 * ones(size(fts,2),1);       % ignore

  % normalize fts so that determinant of each feature is 1 (see notes above)
  scaleA = zeros(size(fts,2),1);
  ftsn   = zeros(size(fts));
  ftsn(1:2,:) = fts(1:2,:);
  for i = 1:size(fts,2)
    a = fts(3,i);    b = fts(4,i);   c = fts(5,i);   A = [a b;b c];
    scaleA(i) = sqrt(det(A));
    ftsn(3:5,i) = fts(3:5,i)./scaleA(i);
  end;

  % set scale for each feature (see note 1 above)
  if size(scale,1) ==1 
    inscale = scale .* scaleA / 5; % divide scale by 5  %!!!
  else 
    inscale = scale .* scaleA / 5;  %!!!
  end;  
  
        
  % -1 (matlab -> C)
  km_in = [ftsn(1,:)'-1 ftsn(2,:)'-1 cornerness inscale angle type lap extr ftsn(3,:)' ftsn(4,:)' ftsn(4,:)' ftsn(5,:)'];

  % save data to input file
  fid = fopen(corner_file,'w');
  fprintf(fid,'%d \n',size(km_in,1)); % save number of features in the file
  fprintf(fid,'0 \n');                % number of descriptors in the file is 0
  fprintf(fid,'%.4f %.4f %d %.3f %.2f %d %d %d %.6f %.6f %.6f %.6f \n' ,km_in');
  fclose(fid);

  % run binary
  exec_string =  [DESCRIPTOR_BIN, ...
                  ' -', descr_type, ...
                  ' -i ', image_file, ...
                  ' -p ', corner_file, ...
                  ' -o ', out_file, ...
                 ];

  [ss,ww] = system(exec_string);

  % load output file
  fid = fopen(out_file,'r');
  if fid==-1
    disp('Could not open output file');
    disp(ss); disp(ww);
    D = []; ftsout=[]; ang=[]; ind=[];
    return;
  end;   
   
  nod = fscanf(fid, '%d',1); % read number of descriptors
  dd  = fscanf(fid, '%d',1); % read dimension of descriptors
  tmp = fscanf(fid, '%f',[dd+12 nod])';
  fclose(fid);

  if verbose
    fprintf('Reading: %d descriptors of dimension %d \n',nod, dd);
  end;

  D = tmp(:,13:end);
  ftsout = [tmp(:,1)+1 tmp(:,2)+1 tmp(:,9) tmp(:,10) tmp(:,12)]';  %+1 C->matlab
  ang = tmp(:,5);

  %find the corresponding indices from ftsout (see notes above)
  [tmpind, d2] = vgg_nearest_neighbour(ftsout(1:2,:), fts(1:2,:));
  ind          = int32(tmpind(:,1));

  % assign input fts to output descriptors (km binary erases affine
  % parameters from fts)
  ftsout = fts(:,ind);

  % delete temporary files
  delete(corner_file);
  delete(image_file);
  delete(out_file);

  %% Set variable final configuration:
  D = uint8(D);
