function [interestPoints,visualWords,regionTypes,angles] = computeVisualWords(img,Clusters,numCenters,HARRIS_AFFINE_BIN,MSER_BIN,DESCRIPTOR_BIN)
% [interestPoints,visualWords,regionTypes,angles] = computeVisualWords(img,Clusters,numCenters,HARRIS_AFFINE_BIN,MSER_BIN,DESCRIPTOR_BIN)
%
% Create visual words.
% 
% Inputs:
% img - Input image.
% Clusters - Cell array of cluster structures.
% numCenters - Number of clusters corresponding to each structure.
% HARRIS_AFFINE_BIN - Path to Harris-Affine interest point binary.
% MSER_BIN - Path to maximally stable extremal regions (MSER) interest
% point binary.
% DESCRIPTOR_BIN - Path to image descriptor binary.
%
% Outputs:
% interestPoints - Extracted interest points, where each column
%   corresponds to [x y a b c].  See computeDescriptors.m for more
%   information.
% visualWords - Vector of visual words.
% regionTypes - The corresponding interest point routine for each feature.
% angles - Dominant orientation of each feature.
%
% Note:  Uses linux binaries 'compute_descriptors.ln' and
% 'detect_points.ln' by Krystian Mikolajczyk (Oxford, now at Univ. of
% Surrey) and 'mser.ln' (MSER region detector) by George Matas (CTU
% Prague).
% 
% Josef Sivic <josef@robots.ox.ac.uk>
% 11/1/2005
% Modified by Bryan Russell <brussell@csail.mit.edu>

img = mean(double(img),3);
img = img./max(img(:));
  
% Get interest points:
[interestPoints,regionTypes] = detectInterestPoints(img,HARRIS_AFFINE_BIN,MSER_BIN);

% Compute descriptors over patch areas
[D,interestPoints,angles,ind] = computeDescriptors(img,interestPoints,DESCRIPTOR_BIN);
regionTypes = uint8(regionTypes(ind));

% Remove small visual words
[D,interestPoints,regionTypes,angles] = removeSmallVisualWords(D,interestPoints,regionTypes,angles,img);
  
% Assign each descriptor to its nearest cluster center
visualWords = VQ_descr(D,interestPoints,regionTypes,Clusters,numCenters);


function [D,interest_points,region_types,angles,ind] = ...
      removeSmallVisualWords(D,interest_points,region_types,angles,img)

  rel_scale_thr = 0.025;
  im_width = size(img,2);
  im_height = size(img,1);
  if ~isempty(interest_points)
    torm = find((gg_fts2sc(interest_points)./sqrt(im_width*im_height))<rel_scale_thr);
    fprintf('%d regions,  %d rem,  %d kept \n',size(interest_points,2),length(torm),size(interest_points,2)-length(torm));
    ind = uint16([1:size(interest_points,2)]);
    D(torm,:) = [];
    interest_points(:,torm) = [];
    region_types(torm) = [];
    angles(torm) = [];
    ind(torm) = [];
  else
    ind = [];
  end


function sc = gg_fts2sc(fts)
  sc = sqrt(fts(3,:).*fts(5,:) - fts(4,:).*fts(4,:));


function [Classes,Dist] = VQ_descr(D,interest_points,region_types,Cl,nw)
% function [Classes,Dist] = VQ_descr(D,interest_points,region_types,cluster_fname)
%
% Assign words to descriptors
%
% josef@robots.ox.ac.uk
% 16/02/2003

  if ~isempty(interest_points)
    D = double(D);
    
    % assign words
    tot_nw = 0;
    Classes = zeros(size(D,1),1);
    Dist = zeros(size(D,1),1);
    for ii = 1:length(Cl)
      if nw(ii) > 0
        r_inds = find(region_types==ii);
        [inds d2] = vgg_nearest_neighbour([D(r_inds,:)]',Cl{ii}.Centers');
        Classes(r_inds,1) = uint16(double(inds) + tot_nw);
        Dist(r_inds,1) = sqrt(d2);
        tot_nw = tot_nw + nw(ii);
      end
    end
  else
    Classes = [];
    Dist = [];
  end
