function [interest_points,region_types] = detectInterestPoints(img,HARRIS_AFFINE_BIN,MSER_BIN)
% [interestPoints,regionTypes] = detectInterestPoints(img,HARRIS_AFFINE_BIN,MSER_BIN)
%
% Compute affine covariant regions for a grayvalued image 'img'.
%
% In:
% img - Input image
% HARRIS_AFFINE_BIN - Path to compiled Harris-Affine binary.
% MSER_BIN - Path to "maximal extremal regions" binary.
%
% Out:
% interestPoints ... [x y a b c], one affine covariant region per column
%     see computeDescriptors.m for help on meaning of [x y a b c]
%
% regionTypes ... 1 = (adam, i.e. Harris-Affine), 2 = (jiri, i.e. MSER)
% 
% Josef Sivic <josef@robots.ox.ac.uk>
% 02/08/2003
% Modified by Bryan Russell <brussell@csail.mit.edu>

  scale_adam = 5;
  scale_jiri = 2;
  mm         = 5;
  ms         = 10; %30;

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % save image on disk  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  tname      = tempname;
  image_file = [tname '.pgm'];
  
  % convert to grayvalues if necessary
  if max(img(:)) > 1 
    img = double(img)./255;
  end
  img = mean(img,3);

  % save image as pgm
  vgg_save_image_pgm(image_file,img*255);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % fast multiscale affine harris (run binary by K. Mikolajczyk)
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  out_file_fmharaff = [tname '.fmharaff'];
   
  % Form the executable name string
  exec_string = [HARRIS_AFFINE_BIN ' -fmharaff' ...
                 ' -i ' image_file ' -o ' out_file_fmharaff];

  % call the binary executable
  [ss,ww] = system(exec_string);
  
  if ss ~= 0
    error('Calling the Krystian binary failed.');
  end
  
  % load output file
  fid = fopen(out_file_fmharaff,'r');
  if fid==-1
    disp('Could not open output file');
    disp(ss); disp(ww);
    fts_a = [];      
  else
    nod     = fscanf(fid, '%f',1); % read 1.0
    unknown = fscanf(fid, '%d',1); % read number of descriptors
    tmp = fscanf(fid, '%f',[12 nod])';
    fclose(fid);
    
    if ~isempty(tmp)
      % shift to MATLAB's (1,1)-based coordinate frame
      fts_a(1:2,:) = tmp(:,1:2)' + 1;
      for j = 1:nod
        A = [tmp(j,9) tmp(j,10); tmp(j,11) tmp(j,12)];
        T = sqrtm(A'*A) * tmp(j,4) * scale_adam;
        fts_a(3:5,j) = [T(1,1) T(1,2) T(2,2)]';
      end   
    end
    delete(out_file_fmharaff);
  end


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % jiri regions (MSER)
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  % run jiri's binary
  out_file_max    = [tname '.mser'];
  pp              = sprintf(' -ms %d -mm %d -t 1 -i ',ms,mm);
  exec_string_max = [MSER_BIN pp image_file ' -o ' out_file_max];
  
  [ss,ww] = system(exec_string_max);
  
  % load regions
  msers = load_mser(out_file_max);
  
  % convert msers into fts format ([x y a b c], one column per region)
  fts_j = [];
  if ~isempty(msers)
    fts_j = gg_mser2adam(msers);
  end
  
  % double the size of the detected jiri regions (standard practice to make the region more distinctive)
  if ~isempty(fts_j)
    fts_js = [fts_j(1:2,:); fts_j(3:5,:)*scale_jiri];
    fts_j = fts_js;
  end

  % delete temporary files
  delete(out_file_max);
  delete(image_file);
  
  % concatenate results
  na = size(fts_a,2);
  nj = size(fts_j,2);
  
  interest_points = [fts_a fts_j];
  region_types    = [ones(na,1)*1; ones(nj,1)*2 ];


% function MSER = load_mser(filename)
%
function MSER = load_mser(filename)

fid = fopen(filename, 'r');
if fid < 0
  error(sprintf('load_mser: File %s does not exist!',filename));
  return;
else
  fclose(fid);
end

A=textread(filename,'%d');

MSER={};
m=1;
i=1;
r=1;
nreg = A(1);
if nreg == 0 % no positive regions
   nreg = A(2);
   i = 3;
   r = 2;
else
   i = 2; 
end;
while i+1<length(A)    
    n              = A(i);
    MSER{m}.border = n;
    MSER{m}.points = reshape(A(i+1:i+2*n),2,n);
    MSER{m}.points = MSER{m}.points+1; % c -> matlab
    i=i+2*n+1;
    if m==nreg & r==1
       nreg = A(i);
       i = i+1;
       r = r+1;
    else   
       m=m+1;        
    end;
end

% function fts = gg_mser2adam(mser) 
% 
% Convert mser into fts format ([x y a b c], one region per column)
%
%
% Josef Sivic <josef@robots.ox.ac.uk

function fts = gg_mser2adam(mser)

% for each reagion
for i = 1:size(mser,2)
    
  % compute moments of the outline      
  
  % 1. zero order moment = length of outline
  M00 = mser{i}.border;
  
  % 2. first order moments = center of mass (com)
  M10 = sum(mser{i}.points(1,:));
  M01 = sum(mser{i}.points(2,:));
  xcm = M10 / M00;
  ycm = M01 / M00;
     
  % 3. second order moments (relative to the center of mass)
  M20 = sum((mser{i}.points(1,:) - xcm).*(mser{i}.points(1,:) - xcm));
  M02 = sum((mser{i}.points(2,:) - ycm).*(mser{i}.points(2,:) - ycm));
  M11 = sum((mser{i}.points(1,:) - xcm).*(mser{i}.points(2,:) - ycm));
  
  M = [M20 M11; M11 M02]/M00;
  
  A = sqrtm(2.0 * M);
  
  % fill in
  fts(:,i) = [xcm+0 ycm+0 A(1,1) A(1,2) A(2,2)]';
end;
