function [pWordsTopics,pTopicsDocuments] = lda(wordsDocuments,Ntopics)
% [pWordsTopics,pTopicsDocuments] = lda(wordsDocuments,Ntopics)
%
% Learning routine for Latent Dirichlet Allocation (Blei et al. 2003).
% This is an implementation of the Gibbs sampling procedure outlined in
% Griffiths and Steyvers 2005.
%
% Inputs:
% wordsDocuments - Word-document histogram matrix.
% Ntopics - Number of topics to discover.
%
% Outputs:
% pWordsTopics - Probability of words given topic.
% pTopicsDocuments - Probability of topics given document.

% Parameters:
%alpha = 0.15;%0.5; alpha = K/Ntopics;
alpha = 15/Ntopics;
beta = 0.5;
Niterations = 100;
termSize = 0.1;

[V,Ndocuments] = size(wordsDocuments);

% Extract vector of "words" and "documents":
[w,d] = find(wordsDocuments);
c = wordsDocuments(find(wordsDocuments));
documents = zeros(sum(c),1);
words = zeros(sum(c),1);
i = 0;
while ~isempty(c)
  documents(i+1:i+length(c)) = d;
  words(i+1:i+length(c)) = w;
  i = i+length(c);
  n = find(c>1);
  c = c(n)-1;
  w = w(n);
  d = d(n);
end

rand('state',sum(100*clock));
z = floor(Ntopics*rand(size(words)))+1;
z_best = z;
L = [];
L_best = [];

nvt = zeros(V,Ntopics); % This should be made sparse
ndt = zeros(Ndocuments,Ntopics); % This should be made sparse
for tt = 1:Ntopics
  idx = find(z==tt);
  nvt(:,tt) = histc(words(idx),1:V);
  ndt(:,tt) = histc(documents(idx),1:Ndocuments);
end

nvt_best = nvt;
ndt_best = ndt;

for jj = 1:Niterations
  fprintf(1,'.');
  [nvt,ndt,z] = ldaResample(words,documents,z,nvt,ndt,alpha,beta);
  L = [L like_mcmc(ndt,alpha,nvt,beta)];
  if isempty(L_best) || (L(end) > L_best(end))
    z_best = z;
    nvt_best = nvt;
    ndt_best = ndt;
    L_best = [L_best L(end)];
  else
    L_best = [L_best L_best(end)];
  end
  show_trace(L,L_best);
  
  %% Early stopping condition
  if (length(L)>2) && (abs(L(end)-L(end-1)) < termSize)
    break;
  end
end

pWordsTopics = phi_mcmc(nvt_best,beta)';
pTopicsDocuments = theta_mcmc(ndt_best,alpha);

function h = show_trace(L,L_best)
  clf;
  plot(L,'r-.');
  hold on;
  plot(L_best,'b-');
  legend('Current Sample','Max Sample',4)
  xlabel('Iteration');
  ylabel('log(p(w|z))');
  drawnow;

function phi = phi_mcmc(nvt,beta)
% PHI = PHI_MCMC(N,BETA) computes the PHI parameters using the word to
% topic assignments contained in N (a VxK size matrix where V is the
% vocabulary size and K are the number of topics) and the hyperparameter
% BETA.  
  
  [V,K] = size(nvt);
  phi = (nvt'+beta)./repmat(sum(nvt,1)'+V*beta,1,V);
  phi = phi./repmat(sum(phi,2),1,V);

function theta = theta_mcmc(ndt,alpha)
% THETA = THETA_MCMC(N,ALPHA) computes the THETA parameters using the
% word to topic assignments contained in N (a DxK size matrix where D is
% the number of documents and K are the number of topics) and the
% hyperparameter ALPHA.  
  
  [D,K] = size(ndt);
  theta = (ndt'+alpha)./repmat(sum(ndt',1)+K*alpha,K,1);
  theta = theta./repmat(sum(theta,1),K,1);

function L = like_mcmc(ndt,alpha,nvt,beta)
% LIKE_MCMC - Computes the log-likelihood for the MCMC solution for LDA.  
% function L = like_mcmc(ndt,alpha,nvt,beta)

  [D,T] = size(ndt);
  [V,T] = size(nvt);
  Nt = sum(nvt,1);
  Nd = sum(ndt,2);
  
  L = D*gammaln(T*alpha)-D*T*gammaln(alpha) + ...
      sum(sum(gammaln(ndt+alpha),2)-gammaln(Nd+T*alpha)) + ...
      T*gammaln(V*beta)-V*T*gammaln(beta) + ...
      sum(sum(gammaln(nvt+beta),1)-gammaln(Nt+V*beta));
