// ldaResample.cpp

#include <time.h>
#include <string.h>
#include <math.h>
#include "mex.h"

int MultiRand(double *p,int dim) {
  double r = ((double)rand() / ((double)(RAND_MAX)+(double)(1)));
  double tot = p[0];
  for(int i = 0; i < dim-1; i++) {
    if(tot>r) return (i+1);
    tot += p[i+1];
  }
  return dim;
}

void ComputeMultinomialParams(int w,int d,int z,double *nvt,
			      double *ndt,double alpha,double beta,
			      int V,int K,int D,double *p,double *Nt,
			      double *Nd) {
  double norm = 0;
  for(int kk = 0; kk < K; kk++) {
    if(kk==(z-1))
      p[kk] = ((nvt[kk*V+w-1]+beta-1)*(ndt[kk*D+d-1]+alpha-1))/
	(Nt[kk]+V*beta-1);
    else
      p[kk] = ((nvt[kk*V+w-1]+beta)*(ndt[kk*D+d-1]+alpha))/
	(Nt[kk]+V*beta);
    norm += p[kk];
  }
  for(int kk = 0; kk < K; kk++) p[kk] /= norm;
}

void InitNtNd(double *ndt,int K,int D,double *Nt,double *Nd) {
  for(int dd = 0; dd < D; dd++) {
    for(int kk = 0; kk < K; kk++) {
      Nd[dd] += ndt[kk*D+dd];
      Nt[kk] += ndt[kk*D+dd];
    }
  }
}

void InitOutputs(double *nvt_in,double *ndt_in,double *z_in,
		 double *nvt,double *ndt,double *z,
		 int N,int V,int K,int D) {
  for(int i = 0; i < V*K; i++) nvt[i] = nvt_in[i];
  for(int i = 0; i < D*K; i++) ndt[i] = ndt_in[i];
  for(int i = 0; i < N; i++) z[i] = z_in[i];
}

void UpdateState(int new_z,double *z,double *w,
		 double *d,double *nvt,double *ndt,
		 int ii,int V,int D,double *Nt) {
  if(((double)new_z)!=z[ii]) {
    nvt[V*((int)z[ii]-1)+(int)w[ii]-1]--;
    ndt[D*((int)z[ii]-1)+(int)d[ii]-1]--;
    nvt[V*(new_z-1)+(int)w[ii]-1]++;
    ndt[D*(new_z-1)+(int)d[ii]-1]++;
    Nt[(int)z[ii]-1]--;
    Nt[new_z-1]++;
    z[ii] = (double)new_z;
  }
}

// Inputs: w,d,z,nvt,ndt,alpha,beta
// Outputs: z_p,nvt,ndt,z
void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[]) {
  srand(time(0));

  if(nrhs != 7) {
    mexErrMsgTxt("Error: 7 args. needed.");
    return;
  }

  double *w = mxGetPr(prhs[0]);
  double *d = mxGetPr(prhs[1]);
  double *z_in = mxGetPr(prhs[2]);
  double *nvt_in = mxGetPr(prhs[3]);
  double *ndt_in = mxGetPr(prhs[4]);
  double alpha = *mxGetPr(prhs[5]);
  double beta = *mxGetPr(prhs[6]);
  int N = mxGetM(prhs[0]);
  int V = mxGetM(prhs[3]);
  int K = mxGetN(prhs[3]);
  int D = mxGetM(prhs[4]);

  // Allocate outputs:
  plhs[0] = mxCreateDoubleMatrix(V,K,mxREAL);
  double *nvt = mxGetPr(plhs[0]);
  plhs[1] = mxCreateDoubleMatrix(D,K,mxREAL);
  double *ndt = mxGetPr(plhs[1]);
  plhs[2] = mxCreateDoubleMatrix(N,1,mxREAL);
  double *z = mxGetPr(plhs[2]);

  // Initialize outputs:
  InitOutputs(nvt_in,ndt_in,z_in,nvt,ndt,z,N,V,K,D);

  // Allocate temporary memory:
  double *Nt = (double*)mxCalloc(K,sizeof(double));
  double *Nd = (double*)mxCalloc(D,sizeof(double));

  //Initialize Nt and Nd:
  InitNtNd(ndt,K,D,Nt,Nd);

  int new_z;
  double *p = (double*)mxCalloc(K,sizeof(double));

  for(int ii=0; ii<N; ii++) {
    // Compute multinomial parameters:
    ComputeMultinomialParams((int)w[ii],(int)d[ii],(int)z[ii],
			     nvt,ndt,alpha,beta,V,K,D,p,Nt,Nd);

    // Sample from the multinomial:
    new_z = MultiRand(p,K);

    // Update state:
    UpdateState(new_z,z,w,d,nvt,ndt,ii,V,D,Nt);
  }

  // Free temporary memory:
  mxFree(Nt);
  mxFree(Nd);
  mxFree(p);
}
